#include <iostream>

class A {
    int ma;  // questo è un membro privato

protected:
    int mb;  // questo è un membro protetto

public:
    int mc;  // questo è un membro pubblico

    A(): A(0, 1, 2) {}

    A(int a, int b, int c): ma{a}, mb{b}, mc{c}
    {}

    inline
    const int& get_ma() const
    {
        return ma;
    }

    inline
    const int& get_mb() const
    {
        return mb;
    }

    inline
    const int& get_mc() const
    {
        return mc;
    }

    friend std::ostream& operator<<(std::ostream& os, const A& a)
    {
        os << "A(" << a.ma
           << "," << a.mb
           << "," << a.mc
           << ")";

        return os;
    }
};

class B : public A {
    int md;  // questo è un membro privato
public:
    B(): A(3,2,1), md{0}
    {}

    inline
    const int& get_md() const
    {
        return md;
    }

    friend std::ostream& operator<<(std::ostream& os, const B& b)
    {
        // b.ma   // ERRORE: ma è privato in A
        std::cout << "B(" 
                  << b.get_ma()
                  << "," << b.mb
                  << "," << b.mc
                  << "," << b.md
                  << ")";

        return os;
    }
};

class C : protected A {
    int md;  // questo è un membro privato
public:
    C(): A(3,2,1), md{0}
    {}

    friend std::ostream& operator<<(std::ostream& os, const C& c)
    {
        // c.ma   // ERRORE: ma è privato in A
        std::cout << "C(" 
                  << c.get_ma()
                  << "," << c.mb
                  << "," << c.mc
                  << "," << c.md
                  << ")";

        return os;
    }
};

class D : private A {
    int md;  // questo è un membro privato
public:
    D(): A(3,2,1), md{0}
    {}

    friend std::ostream& operator<<(std::ostream& os, const D& d)
    {
        // d.ma   // ERRORE: ma è privato in A
        std::cout << "D(" 
                  << d.get_ma()
                  << "," << d.mb
                  << "," << d.mc
                  << "," << d.md
                  << ")";

        return os;
    }
};

int main()
{
    A a;
    std::cout << "a: " << a << std::endl;
    // a.ma   // ERRORE: ma è privato in A
    // a.mb   // ERRORE: mb è protetto in A

    B b;
    std::cout << "b: " << b << std::endl;
    // b.ma   // ERRORE: ma è privato in A
    // b.mb   // ERRORE: mb è protetto in A
    // b.md   // ERRORE: md è privato in B

    C c;
    std::cout << "c: " << c << std::endl;
    // c.ma   // ERRORE: ma è privato in A
    // c.mb   // ERRORE: mb è protetto in A
    // c.mc   // ERRORE: mc è pubblico in A, ma A è una superclasse protetta
    // c.md   // ERRORE: md è privato in C

    D d;
    std::cout << "d: " << d << std::endl;
    // d.ma   // ERRORE: ma è privato in A
    // d.mb   // ERRORE: mb è protetto in A
    // d.mc   // ERRORE: mc è pubblico in A, ma A è una superclasse privata
    // d.md   // ERRORE: md è privato in D
    return 0;
}